@@ -1,6 +1,32 @@
 // Copyright 2006-2008 the V8 project authors. All rights reserved.
+// Redistribution and use in source and binary forms, with or without
+// modification, are permitted provided that the following conditions are
+// met:
+//
+//     * Redistributions of source code must retain the above copyright
+//       notice, this list of conditions and the following disclaimer.
+//     * Redistributions in binary form must reproduce the above
+//       copyright notice, this list of conditions and the following
+//       disclaimer in the documentation and/or other materials provided
+//       with the distribution.
+//     * Neither the name of Google Inc. nor the names of its
+//       contributors may be used to endorse or promote products derived
+//       from this software without specific prior written permission.
+//
+// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include <stdlib.h>
+#include <limits>
 
 #include "cctest.h"
 #include "double-conversion/diy-fp.h"
@@ -13,13 +39,13 @@ using namespace double_conversion;
 
 TEST(Uint64Conversions) {
   // Start by checking the byte-order.
-  uint64_t ordered = UINT64_2PART_C(0x01234567, 89ABCDEF);
+  uint64_t ordered = DOUBLE_CONVERSION_UINT64_2PART_C(0x01234567, 89ABCDEF);
   CHECK_EQ(3512700564088504e-318, Double(ordered).value());
 
-  uint64_t min_double64 = UINT64_2PART_C(0x00000000, 00000001);
+  uint64_t min_double64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x00000000, 00000001);
   CHECK_EQ(5e-324, Double(min_double64).value());
 
-  uint64_t max_double64 = UINT64_2PART_C(0x7fefffff, ffffffff);
+  uint64_t max_double64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x7fefffff, ffffffff);
   CHECK_EQ(1.7976931348623157e308, Double(max_double64).value());
 }
 
@@ -38,22 +64,22 @@ TEST(Uint32Conversions) {
 
 
 TEST(Double_AsDiyFp) {
-  uint64_t ordered = UINT64_2PART_C(0x01234567, 89ABCDEF);
+  uint64_t ordered = DOUBLE_CONVERSION_UINT64_2PART_C(0x01234567, 89ABCDEF);
   DiyFp diy_fp = Double(ordered).AsDiyFp();
   CHECK_EQ(0x12 - 0x3FF - 52, diy_fp.e());
   // The 52 mantissa bits, plus the implicit 1 in bit 52 as a UINT64.
-  CHECK(UINT64_2PART_C(0x00134567, 89ABCDEF) == diy_fp.f());  // NOLINT
+  CHECK(DOUBLE_CONVERSION_UINT64_2PART_C(0x00134567, 89ABCDEF) == diy_fp.f());  // NOLINT
 
-  uint64_t min_double64 = UINT64_2PART_C(0x00000000, 00000001);
+  uint64_t min_double64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x00000000, 00000001);
   diy_fp = Double(min_double64).AsDiyFp();
   CHECK_EQ(-0x3FF - 52 + 1, diy_fp.e());
   // This is a denormal; so no hidden bit.
   CHECK(1 == diy_fp.f());  // NOLINT
 
-  uint64_t max_double64 = UINT64_2PART_C(0x7fefffff, ffffffff);
+  uint64_t max_double64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x7fefffff, ffffffff);
   diy_fp = Double(max_double64).AsDiyFp();
   CHECK_EQ(0x7FE - 0x3FF - 52, diy_fp.e());
-  CHECK(UINT64_2PART_C(0x001fffff, ffffffff) == diy_fp.f());  // NOLINT
+  CHECK(DOUBLE_CONVERSION_UINT64_2PART_C(0x001fffff, ffffffff) == diy_fp.f());  // NOLINT
 }
 
 
@@ -78,32 +104,32 @@ TEST(Single_AsDiyFp) {
 
 
 TEST(AsNormalizedDiyFp) {
-  uint64_t ordered = UINT64_2PART_C(0x01234567, 89ABCDEF);
+  uint64_t ordered = DOUBLE_CONVERSION_UINT64_2PART_C(0x01234567, 89ABCDEF);
   DiyFp diy_fp = Double(ordered).AsNormalizedDiyFp();
   CHECK_EQ(0x12 - 0x3FF - 52 - 11, diy_fp.e());
-  CHECK((UINT64_2PART_C(0x00134567, 89ABCDEF) << 11) ==
+  CHECK((DOUBLE_CONVERSION_UINT64_2PART_C(0x00134567, 89ABCDEF) << 11) ==
         diy_fp.f());  // NOLINT
 
-  uint64_t min_double64 = UINT64_2PART_C(0x00000000, 00000001);
+  uint64_t min_double64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x00000000, 00000001);
   diy_fp = Double(min_double64).AsNormalizedDiyFp();
   CHECK_EQ(-0x3FF - 52 + 1 - 63, diy_fp.e());
   // This is a denormal; so no hidden bit.
-  CHECK(UINT64_2PART_C(0x80000000, 00000000) == diy_fp.f());  // NOLINT
+  CHECK(DOUBLE_CONVERSION_UINT64_2PART_C(0x80000000, 00000000) == diy_fp.f());  // NOLINT
 
-  uint64_t max_double64 = UINT64_2PART_C(0x7fefffff, ffffffff);
+  uint64_t max_double64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x7fefffff, ffffffff);
   diy_fp = Double(max_double64).AsNormalizedDiyFp();
   CHECK_EQ(0x7FE - 0x3FF - 52 - 11, diy_fp.e());
-  CHECK((UINT64_2PART_C(0x001fffff, ffffffff) << 11) ==
+  CHECK((DOUBLE_CONVERSION_UINT64_2PART_C(0x001fffff, ffffffff) << 11) ==
         diy_fp.f());  // NOLINT
 }
 
 
 TEST(Double_IsDenormal) {
-  uint64_t min_double64 = UINT64_2PART_C(0x00000000, 00000001);
+  uint64_t min_double64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x00000000, 00000001);
   CHECK(Double(min_double64).IsDenormal());
-  uint64_t bits = UINT64_2PART_C(0x000FFFFF, FFFFFFFF);
+  uint64_t bits = DOUBLE_CONVERSION_UINT64_2PART_C(0x000FFFFF, FFFFFFFF);
   CHECK(Double(bits).IsDenormal());
-  bits = UINT64_2PART_C(0x00100000, 00000000);
+  bits = DOUBLE_CONVERSION_UINT64_2PART_C(0x00100000, 00000000);
   CHECK(!Double(bits).IsDenormal());
 }
 
@@ -122,7 +148,7 @@ TEST(Double_IsSpecial) {
   CHECK(Double(Double::Infinity()).IsSpecial());
   CHECK(Double(-Double::Infinity()).IsSpecial());
   CHECK(Double(Double::NaN()).IsSpecial());
-  uint64_t bits = UINT64_2PART_C(0xFFF12345, 00000000);
+  uint64_t bits = DOUBLE_CONVERSION_UINT64_2PART_C(0xFFF12345, 00000000);
   CHECK(Double(bits).IsSpecial());
   // Denormals are not special:
   CHECK(!Double(5e-324).IsSpecial());
@@ -172,7 +198,7 @@ TEST(Double_IsInfinite) {
   CHECK(!Double(-0.0).IsInfinite());
   CHECK(!Double(1.0).IsInfinite());
   CHECK(!Double(-1.0).IsInfinite());
-  uint64_t min_double64 = UINT64_2PART_C(0x00000000, 00000001);
+  uint64_t min_double64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x00000000, 00000001);
   CHECK(!Double(min_double64).IsInfinite());
 }
 
@@ -192,7 +218,7 @@ TEST(Single_IsInfinite) {
 
 TEST(Double_IsNan) {
   CHECK(Double(Double::NaN()).IsNan());
-  uint64_t other_nan = UINT64_2PART_C(0xFFFFFFFF, 00000001);
+  uint64_t other_nan = DOUBLE_CONVERSION_UINT64_2PART_C(0xFFFFFFFF, 00000001);
   CHECK(Double(other_nan).IsNan());
   CHECK(!Double(Double::Infinity()).IsNan());
   CHECK(!Double(-Double::Infinity()).IsNan());
@@ -200,7 +226,7 @@ TEST(Double_IsNan) {
   CHECK(!Double(-0.0).IsNan());
   CHECK(!Double(1.0).IsNan());
   CHECK(!Double(-1.0).IsNan());
-  uint64_t min_double64 = UINT64_2PART_C(0x00000000, 00000001);
+  uint64_t min_double64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x00000000, 00000001);
   CHECK(!Double(min_double64).IsNan());
 }
 
@@ -226,7 +252,7 @@ TEST(Double_Sign) {
   CHECK_EQ(-1, Double(-Double::Infinity()).Sign());
   CHECK_EQ(1, Double(0.0).Sign());
   CHECK_EQ(-1, Double(-0.0).Sign());
-  uint64_t min_double64 = UINT64_2PART_C(0x00000000, 00000001);
+  uint64_t min_double64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x00000000, 00000001);
   CHECK_EQ(1, Double(min_double64).Sign());
 }
 
@@ -264,7 +290,7 @@ TEST(Double_NormalizedBoundaries) {
   CHECK((1 << 9) == diy_fp.f() - boundary_minus.f());  // NOLINT
   CHECK((1 << 10) == boundary_plus.f() - diy_fp.f());  // NOLINT
 
-  uint64_t min_double64 = UINT64_2PART_C(0x00000000, 00000001);
+  uint64_t min_double64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x00000000, 00000001);
   diy_fp = Double(min_double64).AsNormalizedDiyFp();
   Double(min_double64).NormalizedBoundaries(&boundary_minus, &boundary_plus);
   CHECK_EQ(diy_fp.e(), boundary_minus.e());
@@ -276,7 +302,7 @@ TEST(Double_NormalizedBoundaries) {
   CHECK((static_cast<uint64_t>(1) << 62) ==
         diy_fp.f() - boundary_minus.f());  // NOLINT
 
-  uint64_t smallest_normal64 = UINT64_2PART_C(0x00100000, 00000000);
+  uint64_t smallest_normal64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x00100000, 00000000);
   diy_fp = Double(smallest_normal64).AsNormalizedDiyFp();
   Double(smallest_normal64).NormalizedBoundaries(&boundary_minus,
                                                  &boundary_plus);
@@ -287,7 +313,7 @@ TEST(Double_NormalizedBoundaries) {
   CHECK(diy_fp.f() - boundary_minus.f() == boundary_plus.f() - diy_fp.f());
   CHECK((1 << 10) == diy_fp.f() - boundary_minus.f());  // NOLINT
 
-  uint64_t largest_denormal64 = UINT64_2PART_C(0x000FFFFF, FFFFFFFF);
+  uint64_t largest_denormal64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x000FFFFF, FFFFFFFF);
   diy_fp = Double(largest_denormal64).AsNormalizedDiyFp();
   Double(largest_denormal64).NormalizedBoundaries(&boundary_minus,
                                                   &boundary_plus);
@@ -296,7 +322,7 @@ TEST(Double_NormalizedBoundaries) {
   CHECK(diy_fp.f() - boundary_minus.f() == boundary_plus.f() - diy_fp.f());
   CHECK((1 << 11) == diy_fp.f() - boundary_minus.f());  // NOLINT
 
-  uint64_t max_double64 = UINT64_2PART_C(0x7fefffff, ffffffff);
+  uint64_t max_double64 = DOUBLE_CONVERSION_UINT64_2PART_C(0x7fefffff, ffffffff);
   diy_fp = Double(max_double64).AsNormalizedDiyFp();
   Double(max_double64).NormalizedBoundaries(&boundary_minus, &boundary_plus);
   CHECK_EQ(diy_fp.e(), boundary_minus.e());
@@ -398,7 +424,7 @@ TEST(NextDouble) {
   CHECK_EQ(4e-324, d2.NextDouble());
   CHECK_EQ(-1.7976931348623157e308, Double(-Double::Infinity()).NextDouble());
   CHECK_EQ(Double::Infinity(),
-           Double(UINT64_2PART_C(0x7fefffff, ffffffff)).NextDouble());
+           Double(DOUBLE_CONVERSION_UINT64_2PART_C(0x7fefffff, ffffffff)).NextDouble());
 }
 
 
@@ -417,5 +443,21 @@ TEST(PreviousDouble) {
   CHECK_EQ(-4e-324, d2.PreviousDouble());
   CHECK_EQ(1.7976931348623157e308, Double(Double::Infinity()).PreviousDouble());
   CHECK_EQ(-Double::Infinity(),
-           Double(UINT64_2PART_C(0xffefffff, ffffffff)).PreviousDouble());
+           Double(DOUBLE_CONVERSION_UINT64_2PART_C(0xffefffff, ffffffff)).PreviousDouble());
+}
+
+TEST(SignalingNan) {
+  Double nan(Double::NaN());
+  CHECK(nan.IsNan());
+  CHECK(nan.IsQuietNan());
+  CHECK(Double(std::numeric_limits<double>::quiet_NaN()).IsQuietNan());
+  CHECK(Double(std::numeric_limits<double>::signaling_NaN()).IsSignalingNan());
+}
+
+TEST(SignalingNanSingle) {
+  Single nan(Single::NaN());
+  CHECK(nan.IsNan());
+  CHECK(nan.IsQuietNan());
+  CHECK(Single(std::numeric_limits<float>::quiet_NaN()).IsQuietNan());
+  CHECK(Single(std::numeric_limits<float>::signaling_NaN()).IsSignalingNan());
 }
